local AceGUI = LibStub("AceGUI-3.0")

-- WoW APIs
local CreateFrame, UIParent = CreateFrame, UIParent

-- Global vars/functions that we don't upvalue since they might get hooked, or upgraded
-- List them here for Mikk's FindGlobals script
-- GLOBALS: GameFontHighlightSmall

----------------------------------------------
-- PentaTextInteractiveLabel 	 			--
----------------------------------------------
do
	local Type = "PentaTextInteractiveLabel"
	local Version = 1

	local function OnAcquire(self)
		self:SetHeight(18)
		self:SetWidth(200)
		self:SetTextWidths(40, 40, 40, 40)
		self:SetText()
		self:SetColor()
		self:SetFontObject()
		self:SetHighlight()
		self:SetHighlightTexCoord()
		self:UnlockHighlight()
	end

	local function OnRelease(self)
		self:SetDisabled(false)
		self.frame:ClearAllPoints()
		self.frame:Hide()
	end

	local function SetText(self, t1, t2, t3, t4, t5)
		self.textlabels[1]:SetText(t1 or '')
		self.textlabels[2]:SetText(t2 or '')
		self.textlabels[3]:SetText(t3 or '')
		self.textlabels[4]:SetText(t4 or '')
		self.textlabels[5]:SetText(t5 or '')
	end

	local function SetColor(self, r1, g1, b1, r2, g2, b2, r3, g3, b3, r4, g4, b4, r5, g5, b5)
		if not (r1 and g1 and b1) then r1, g1, b1 = 1, 1, 1 end
		self.textlabels[1]:SetVertexColor(r1, g1, b1)
		if not (r2 and g2 and b2) then r2, g2, b2 = 1, 1, 1 end
		self.textlabels[2]:SetVertexColor(r2, g2, b2)
		if not (r3 and g3 and b3) then r3, g3, b3 = 1, 1, 1 end
		self.textlabels[3]:SetVertexColor(r3, g3, b3)
		if not (r4 and g4 and b4) then r4, g4, b4 = 1, 1, 1 end
		self.textlabels[4]:SetVertexColor(r4, g4, b4)
		if not (r5 and g5 and b5) then r5, g5, b5 = 1, 1, 1 end
		self.textlabels[5]:SetVertexColor(r5, g5, b5)
	end

	local function OnWidthSet(self, width)
		if self.resizing then return end
	end

	local function SetFont(self, font, height, flags)
		self.textlabels[1]:SetFont(font, height, flags)
		self.textlabels[2]:SetFont(font, height, flags)
		self.textlabels[3]:SetFont(font, height, flags)
		self.textlabels[4]:SetFont(font, height, flags)
		self.textlabels[5]:SetFont(font, height, flags)
	end

	local function SetFontObject(self, font)
		self.textlabels[1]:SetFontObject(font or GameFontHighlightSmall)
		self.textlabels[2]:SetFontObject(font or GameFontHighlightSmall)
		self.textlabels[3]:SetFontObject(font or GameFontHighlightSmall)
		self.textlabels[4]:SetFontObject(font or GameFontHighlightSmall)
		self.textlabels[5]:SetFontObject(font or GameFontHighlightSmall)
	end

	local function SetHighlight(self, ...)
		self.highlight:SetTexture(...)
	end

	local function SetHighlightTexCoord(self, ...)
		if (select('#', ...) >= 1) then
			self.highlight:SetTexCoord(...)
		else
			self.highlight:SetTexCoord(0, 1, 0, 1)
		end
	end

	local function SetDisabled(self,disabled)
		self.disabled = disabled
		if disabled then
			self.frame:EnableMouse(false)
			for x = 1, 5 do
				self.textlabels[x]:SetTextColor(0.5, 0.5, 0.5)
			end
		else
			self.frame:EnableMouse(true)
			for x = 1, 5 do
				self.textlabels[x]:SetTextColor(1, 1, 1)
			end
		end
	end

	local function SetTextWidths(self, a, b, c, d)
		if (not (a and b and c and d)) then
			a, b, c, d = 40, 40, 40, 40
		end
		self.textlabels[1]:SetWidth(a)
		self.textlabels[2]:SetWidth(b)
		self.textlabels[3]:SetWidth(c)
		self.textlabels[4]:SetWidth(d)
	end

	local function OnEnter(self)
		self.obj.highlight:Show()
		self.obj:Fire("OnEnter")
	end

	local function OnLeave(self)
		if (not self.obj.lockHighlight) then
			self.obj.highlight:Hide()
		end
		self.obj:Fire("OnLeave")
	end

	local function OnClick(self, ...)
		self.obj:Fire("OnClick", ...)

		if (self.obj.lockHighlight) then
			self.obj.lockHighlight = false
			self.obj.highlight:Hide()
		else
			self.obj.lockHighlight = true
			self.obj.highlight:Show()
		end

		AceGUI:ClearFocus()
	end

	local function UnlockHighlight(self)
		self.lockHighlight = false
		self.highlight:Hide()
	end

	local function IsSelected(self)
		if (self.lockHighlight) then
			return true
		else
			return false
		end
	end

	local function Constructor()
		local frame = CreateFrame("Frame",nil,UIParent)
		local self = {}
		self.type = Type

		frame:EnableMouse(true)
		frame:SetScript("OnEnter", OnEnter)
		frame:SetScript("OnLeave", OnLeave)
		frame:SetScript("OnMouseDown", OnClick)

		self.OnRelease = OnRelease
		self.OnAcquire = OnAcquire
		self.SetText = SetText
		self.SetColor = SetColor
		self.frame = frame
		self.OnWidthSet = OnWidthSet
		self.SetFont = SetFont
		self.SetFontObject = SetFontObject
		self.SetHighlight = SetHighlight
		self.SetHighlightTexCoord = SetHighlightTexCoord
		self.SetDisabled = SetDisabled
		self.SetTextWidths = SetTextWidths
		self.UnlockHighlight = UnlockHighlight
		self.IsSelected = IsSelected
		frame.obj = self

		frame:SetHeight(18)
		frame:SetWidth(200)

		self.textlabels = {}

		local label
		for x = 1, 5 do
			label = frame:CreateFontString(nil, 'BACKGROUND', 'GameFontHighlightSmall')
			label:SetWidth(40)
			label:SetJustifyH('LEFT')
			label:SetJustifyV('MIDDLE')
			tinsert(self.textlabels, label)
		end

		self.textlabels[1]:SetPoint('LEFT', frame, 'LEFT', 0, 0)
		self.textlabels[2]:SetPoint('LEFT', self.textlabels[1], 'RIGHT')
		self.textlabels[3]:SetPoint('LEFT', self.textlabels[2], 'RIGHT')
		self.textlabels[4]:SetPoint('LEFT', self.textlabels[3], 'RIGHT')
		self.textlabels[5]:SetPoint('LEFT', self.textlabels[4], 'RIGHT')
		self.textlabels[5]:SetPoint('RIGHT', frame, 'RIGHT', 0 ,0)

		local highlight = frame:CreateTexture(nil, "OVERLAY")
		highlight:SetTexture(nil)
		highlight:SetAllPoints()
		highlight:SetBlendMode("ADD")
		highlight:Hide()
		self.highlight = highlight
		self.lockHighlight = false

		AceGUI:RegisterAsWidget(self)
		return self
	end

	AceGUI:RegisterWidgetType(Type,Constructor,Version)
end

